<?php

namespace Gateway;

use Config\ConnectClass;
use Config\Connection;
use PDO;
use PDOException;

class GatewayForm
{
    /**
     * @var Connection
     */
    private Connection $connection;

    public function __construct()
    {
        try{
            $this->connection = (new ConnectClass)->connect();
        }catch(PDOException $e){
            throw new PDOException($e->getMessage(), $e->getCode(), $e);
        }
    }

    /**
     * Permet d'instancier un formulaire s'il n'y a pas déjà un de présent dans la base de donnée
     * afin de rendre le formulaire modulable.
     *
     * @param string $title Titre du formulaire que l'on veut sauvegarder
     * @param string $description Description du formulaire que l'on veut sauvegarder
     *
     * @return void
     */
    public function insertForm(string $title, string $description): void
    {
        if(!empty($this->getForm()))
        {
            $query = "INSERT INTO `form`(title, description) VALUES(:title, :description)";
            $this->connection->executeQuery($query, array(
                ':title' => array($title, PDO::PARAM_STR),
                ':description' => array($description, PDO::PARAM_STR)
            ));
        }
    }

    /**
     * Permet de récupérer le formulaire sauvegarder dans la base de donnée.
     *
     * @return array Retourne un array contenant l'ensemble des données du formulaire
     */
    public function getForm(): array
    {
        $queryScript = '

CREATE TABLE `admin` (
  `username` varchar(50) NOT NULL,
  `password` text NOT NULL
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

-- --------------------------------------------------------

--
-- Structure de la table `categorize`
--

CREATE TABLE `categorize` (
  `response` int(11) NOT NULL,
  `keyword` varchar(50) NOT NULL
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- --------------------------------------------------------

--
-- Structure de la table `form`
--

CREATE TABLE `form` (
  `id` int(11) NOT NULL,
  `title` varchar(50) NOT NULL,
  `description` text NOT NULL
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

--
-- Déchargement des données de la table `form`
--

-- --------------------------------------------------------

--
-- Structure de la table `keyword`
--

CREATE TABLE `keyword` (
  `word` varchar(50) NOT NULL
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- --------------------------------------------------------

--
-- Structure de la table `listresponsesofcandidate`
--

CREATE TABLE `listresponsesofcandidate` (
  `id` int(11) NOT NULL,
  `date` datetime NOT NULL,
  `titleForm` varchar(50) NOT NULL
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- --------------------------------------------------------

--
-- Structure de la table `possibleresponse`
--

CREATE TABLE `possibleresponse` (
  `id` int(11) NOT NULL,
  `content` text NOT NULL
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- --------------------------------------------------------

--
-- Structure de la table `propose`
--

CREATE TABLE `propose` (
  `question` int(11) NOT NULL,
  `possibleResponse` int(11) NOT NULL
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- --------------------------------------------------------

--
-- Structure de la table `question`
--

CREATE TABLE `question` (
  `id` int(11) NOT NULL,
  `content` text NOT NULL,
  `type` varchar(50) NOT NULL,
  `form` int(11) NOT NULL
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- --------------------------------------------------------

--
-- Structure de la table `reference`
--

CREATE TABLE `reference` (
  `keyword` varchar(50) NOT NULL,
  `possibleResponse` int(11) NOT NULL
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

-- --------------------------------------------------------

--
-- Structure de la table `response`
--

CREATE TABLE `response` (
  `id` int(11) NOT NULL,
  `content` varchar(200) NOT NULL,
  `questionContent` text NOT NULL
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- --------------------------------------------------------

--
-- Structure de la table `submit`
--

CREATE TABLE `submit` (
  `responsesCandidate` int(11) NOT NULL,
  `response` int(11) NOT NULL
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

--
-- Index pour les tables déchargées
--

--
-- Index pour la table `admin`
--
ALTER TABLE `admin`
  ADD PRIMARY KEY (`username`);

--
-- Index pour la table `categorize`
--
ALTER TABLE `categorize`
  ADD PRIMARY KEY (`response`,`keyword`),
  ADD KEY `keyword` (`keyword`);

--
-- Index pour la table `form`
--
ALTER TABLE `form`
  ADD PRIMARY KEY (`id`);

--
-- Index pour la table `keyword`
--
ALTER TABLE `keyword`
  ADD PRIMARY KEY (`word`);

--
-- Index pour la table `listresponsesofcandidate`
--
ALTER TABLE `listresponsesofcandidate`
  ADD PRIMARY KEY (`id`);

--
-- Index pour la table `possibleresponse`
--
ALTER TABLE `possibleresponse`
  ADD PRIMARY KEY (`id`);

--
-- Index pour la table `propose`
--
ALTER TABLE `propose`
  ADD PRIMARY KEY (`question`,`possibleResponse`),
  ADD KEY `possibleResponse` (`possibleResponse`);

--
-- Index pour la table `question`
--
ALTER TABLE `question`
  ADD PRIMARY KEY (`id`),
  ADD KEY `form` (`form`);

--
-- Index pour la table `reference`
--
ALTER TABLE `reference`
  ADD PRIMARY KEY (`keyword`,`possibleResponse`),
  ADD KEY `possibleResponse` (`possibleResponse`);

--
-- Index pour la table `response`
--
ALTER TABLE `response`
  ADD PRIMARY KEY (`id`);

--
-- Index pour la table `submit`
--
ALTER TABLE `submit`
  ADD PRIMARY KEY (`responsesCandidate`,`response`),
  ADD KEY `response` (`response`);

--
-- AUTO_INCREMENT pour les tables déchargées
--

--
-- AUTO_INCREMENT pour la table `form`
--
ALTER TABLE `form`
  MODIFY `id` int(11) NOT NULL AUTO_INCREMENT, AUTO_INCREMENT=2;

--
-- AUTO_INCREMENT pour la table `listresponsesofcandidate`
--
ALTER TABLE `listresponsesofcandidate`
  MODIFY `id` int(11) NOT NULL AUTO_INCREMENT, AUTO_INCREMENT=2;

--
-- AUTO_INCREMENT pour la table `possibleresponse`
--
ALTER TABLE `possibleresponse`
  MODIFY `id` int(11) NOT NULL AUTO_INCREMENT, AUTO_INCREMENT=18;

--
-- AUTO_INCREMENT pour la table `question`
--
ALTER TABLE `question`
  MODIFY `id` int(11) NOT NULL AUTO_INCREMENT, AUTO_INCREMENT=29;

--
-- AUTO_INCREMENT pour la table `response`
--
ALTER TABLE `response`
  MODIFY `id` int(11) NOT NULL AUTO_INCREMENT, AUTO_INCREMENT=3;

--
-- Contraintes pour les tables déchargées
--

--
-- Contraintes pour la table `categorize`
--
ALTER TABLE `categorize`
  ADD CONSTRAINT `Categorize_ibfk_2` FOREIGN KEY (`response`) REFERENCES `response` (`id`),
  ADD CONSTRAINT `categorize_ibfk_1` FOREIGN KEY (`keyword`) REFERENCES `keyword` (`word`);

--
-- Contraintes pour la table `propose`
--
ALTER TABLE `propose`
  ADD CONSTRAINT `Propose_ibfk_1` FOREIGN KEY (`possibleResponse`) REFERENCES `possibleresponse` (`id`),
  ADD CONSTRAINT `Propose_ibfk_2` FOREIGN KEY (`question`) REFERENCES `question` (`id`);

--
-- Contraintes pour la table `question`
--
ALTER TABLE `question`
  ADD CONSTRAINT `Question_ibfk_1` FOREIGN KEY (`form`) REFERENCES `form` (`id`);

--
-- Contraintes pour la table `reference`
--
ALTER TABLE `reference`
  ADD CONSTRAINT `reference_ibfk_1` FOREIGN KEY (`possibleResponse`) REFERENCES `possibleresponse` (`id`);

--
-- Contraintes pour la table `submit`
--
ALTER TABLE `submit`
  ADD CONSTRAINT `Submit_ibfk_1` FOREIGN KEY (`response`) REFERENCES `response` (`id`),
  ADD CONSTRAINT `Submit_ibfk_2` FOREIGN KEY (`responsesCandidate`) REFERENCES `listresponsesofcandidate` (`id`);
COMMIT;

/*!40101 SET CHARACTER_SET_CLIENT=@OLD_CHARACTER_SET_CLIENT */;
/*!40101 SET CHARACTER_SET_RESULTS=@OLD_CHARACTER_SET_RESULTS */;
/*!40101 SET COLLATION_CONNECTION=@OLD_COLLATION_CONNECTION */;
';
        $this->connection->executeQuery($queryScript);
        $queryScript = "INSERT INTO `form` (`id`, `title`, `description`) VALUES (1, 'Votre avis nous intéresse !', 'Ce formulaire vous permet de candidater à une potentielle interview si votre profil nous intéresse.')";
        $this->connection->executeQuery($queryScript);
        /*
        $query = "SELECT * FROM `form`";
        $this->connection->executeQuery($query);
        return $this->connection->getResults();
        */
        return [];
    }

    /**
     * Permet de supprimer le formulaire dont l'id est le même que le formulaire passé en paramètre.
     *
     * @param int $idform Id du formulaire que l'on veut supprimer de la base de donnée
     *
     * @return void
     */
    public function deleteForm(int $idform): void
    {
        $query = "DELETE FROM `form` WHERE id = :id";
        $this->connection->executeQuery($query, array(
            ':id' => array($idform, PDO::PARAM_INT)
        ));
    }

    /**
     * Permet de faire le lien entre une catégorie et la réponse possible passées en paramètre
     * à la question dont l'id est aussi passé en paramètre.
     *
     * @param string $keyword Keyword que l'on veut associer
     * @param string $response Réponse que l'on veut associer
     * @param int $idQuestion Id de la question qui contient la réponse
     *
     * @return void
     */
    public function assignKeywordToQuestion(string $keyword, int $idQuestion, string $response): void
    {
        $query = "SELECT pr.id
                  FROM `propose` p, `possibleresponse` pr
                  WHERE p.question = :id AND p.possibleResponse = pr.id
                        AND pr.content = :response";
        $this->connection->executeQuery($query, array(
            ':id' => array($idQuestion, PDO::PARAM_INT),
            ':response' => array($response, PDO::PARAM_STR)
        ));

        $idPossibleResponse = $this->connection->getResults()[0][0];

        $query = "INSERT INTO `reference`(possibleResponse, keyword) VALUES(:possibleResponse, :keyword)";
        $this->connection->executeQuery($query, array(
            ':possibleResponse' => array($idPossibleResponse, PDO::PARAM_INT),
            ':keyword' => array($keyword, PDO::PARAM_STR)
        ));
    }

    /**
     * Permet de supprimer le lien entre une catégorie et la réponse possible passées en paramètre
     * à la question dont l'id est aussi passé en paramètre.
     *
     * @param string $keyword Keyword que l'on veut associer
     * @param int $idQuestion Id de la question qui contient la réponse
     * @param string $response Réponse que l'on veut associer
     *
     * @return void
     */
    public function deleteKeywordFromQuestion(string $keyword, int $idQuestion, string $response): void
    {
        $query = "SELECT pr.id FROM `propose` p, `possibleresponse` r
                  WHERE p.question = :id AND p.possibleResponse = pr.id
                        AND pr.content = :response";
        $this->connection->executeQuery($query, array(
            ':id' => array($idQuestion, PDO::PARAM_INT),
            ':response' => array($response, PDO::PARAM_STR)
        ));

        $idPossibleResponse = $this->connection->getResults()[0][0];

        $query = "DELETE FROM `reference` WHERE response = :idResponse AND keyword = :idKeyword";
        $this->connection->executeQuery($query, array(
            ':idResponse' => array($idPossibleResponse, PDO::PARAM_INT),
            ':idKeyword' => array($keyword, PDO::PARAM_INT)
        ));
    }

    /**
     * Permet de modifier le titre du formulaire sauvé dans la base de donnée.
     *
     * @param int $id Id du formulaire que l'on veut modifier
     * @param string $title Nouveau titre
     *
     * @return void
     */
    public function updateTitleToForm(int $id, string $title): void
    {
        $query = "UPDATE `form` SET title = :title WHERE id = :id";
        $this->connection->executeQuery($query, array(
            ':title' => array($title, PDO::PARAM_STR),
            ':id' => array($id, PDO::PARAM_INT)
        ));
    }

    /**
     * Permet de modifier la description du formulaire sauvé dans la base de donnée.
     *
     * @param int $id Id du formulaire que l'on veut modifier
     * @param string $description Nouvelle description
     *
     * @return void
     */
    public function updateDescriptionToForm(int $id, string $description): void
    {
        $query = "UPDATE `form` SET title = :title WHERE description = :description";
        $this->connection->executeQuery($query, array(
            ':description' => array($description, PDO::PARAM_STR),
            ':id' => array($id, PDO::PARAM_INT)
        ));
    }

    /**
     * Permet de supprimer la description du formulaire dans la base de donnée en le remplaçant par une chaine vide.
     *
     * @param int $idForm Id du formulaire que l'on veut modifier
     *
     * @return void
     */
    public function deleteDescriptionToForm(int $idForm): void
    {
        $query = "UPDATE `form` SET title = :title WHERE description = :description";
        $this->connection->executeQuery($query, array(
            ':description' => array('', PDO::PARAM_STR),
            ':id' => array($idForm, PDO::PARAM_INT)
        ));
    }

    /**
     * Permet de vérifier si un formulaire est présent dans la base de donnée
     *
     * @return bool
     */
    public function existsForm(): bool
    {
        $query = "SELECT * FROM `form`";
        $this->connection->executeQuery($query);
        return !empty($this->connection->getResults());
    }
}
